/*****************************************************************************
 *  Rohde&Schwarz NGx Power Supply instrument driver
 *  VXIpnp Instrument Driver
 *  Original Release: February 2019
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *****************************************************************************/
#include "rsngx.h"

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString statDataArr[] = {"VOLT", "CURR", "POW", NULL};

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsngx_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsngx_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_init(ViRsrc resourceName, ViBoolean IDQuery,
                             ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsngx_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsngx_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr *)rsngx_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsngx", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsngx_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMA180", VI_FALSE: "CMA" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSNGX_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.5.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 5, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    // Here perform settings that are default for this driver, but can be overwritten by the optionsString settings
    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsngx_RsCoreRepCapTable, rsngx_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "SMW200A", false : "SMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSNGX_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSNGX_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsngx_reset(instrSession));
    }
    else
    {
        checkErr(rsngx_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSNGX_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rsngx_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN  Sets the voltage and current value of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltage/Sets the voltage.
/// HIPAR current/Sets the current.
ViStatus _VI_FUNC rsngx_ConfigureOutput(ViSession instrSession,
                                        ViInt32   channel,
                                        ViReal64  voltage,
                                        ViReal64  current)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "APPL %.12lG,%.12lG,OUT%ld", voltage, current, channel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the currently voltage-current measurement pair.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltage/Returns the currently measured voltage value.
/// HIPAR current/Returns the currently measured current value.
ViStatus _VI_FUNC rsngx_ReadVoltageCurrentPair(ViSession instrSession,
                                               ViInt32   channel,
                                               ViReal64* voltage,
                                               ViReal64* current)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* instrumentResponse = NULL;
    ViInt32   arraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(RsCore_GetAttributeViStringAsViReal64Array(instrSession, NULL, RSNGX_ATTR_HIDDEN_READ_VOLTAGE_CURRENT_PAIR, NULL, &instrumentResponse, &arraySize));

    *voltage = instrumentResponse[0];
    *current = instrumentResponse[1];

Error:
    if (instrumentResponse) free(instrumentResponse);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the voltage value of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltageLevel/Sets/Queries the voltage value of the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureVoltageLevel(ViSession instrSession,
                                              ViInt32   channel,
                                              ViReal64  voltageLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_LEVEL, voltageLevel),
            3, "Voltage Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the range for target voltage.
/// HIFN
/// HIFN     Refer the datasheet for the measurement accuracy according to the
/// HIFN  measurement range selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR range/Sets or queries the range for target voltage.
ViStatus _VI_FUNC rsngx_ConfigureVoltageRange(ViSession instrSession,
                                              ViInt32   channel,
                                              ViReal64  range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_VOLTAGE_RANGE, range),
            3, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the safety limits for voltage.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR safetyLimitState/Activates/Deactivates the safety limit for voltage.
/// HIPAR lowerSafetyLimit/Sets the lower safety limit for voltage.
/// HIPAR upperSafetyLimit/Sets the upper safety limit for voltage.
ViStatus _VI_FUNC rsngx_ConfigureVoltageSafetyLimit(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViBoolean safetyLimitState,
                                                    ViReal64  lowerSafetyLimit,
                                                    ViReal64  upperSafetyLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_VOLTAGE_SAFETY_LIMIT_ENABLED, safetyLimitState),
            3, "Safety Limit State");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_SAFETY_LIMIT_LOWER, lowerSafetyLimit),
            4, "Lower Safety Limit");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_SAFETY_LIMIT_UPPER, upperSafetyLimit),
            5, "Upper Safety Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the negative current value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltage/Sets the negative current value.
/// HIPAR voltageValue/Sets the negative current value.
ViStatus _VI_FUNC rsngx_ConfigureVoltageCurrentNegative(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   voltage,
                                                        ViReal64  voltageValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_VOLTAGE_CURRENT_NEGATIVE, voltage),
            3, "Voltage");

    if (voltage == RSNGX_VAL_VOLTAGE_CURRENT_NEGATIVE_VAL)
    {
        viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_CURRENT_NEGATIVE_VALUE, voltageValue),
                4, "Voltage Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the negative value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltage/Sets the negative value.
/// HIPAR voltageValue/Sets the negative value.
ViStatus _VI_FUNC rsngx_ConfigureVoltageNegative(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   voltage,
                                                 ViReal64  voltageValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_VOLTAGE_NEGATIVE, voltage),
            3, "Voltage");

    if (voltage == RSNGX_VAL_VOLTAGE_CURRENT_NEGATIVE_VAL)
    {
        viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_NEGATIVE_VALUE, voltageValue),
                4, "Voltage Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method returns the maximum programmable voltage level that the
/// HIFN  power supply accepts for a particular current limit on an output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR currentLevel/The current limit for which to determine the maximum programmable
/// HIPAR currentLevel/voltage level.
/// HIPAR maxVoltageLevel/Returns the maximum programmable voltage level for the specified
/// HIPAR maxVoltageLevel/current level.
ViStatus _VI_FUNC rsngx_QueryMaxVoltageLevel(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  currentLevel,
                                             ViReal64* maxVoltageLevel)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 old_current_level = 0.0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_LEVEL, &old_current_level));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_LEVEL, currentLevel),
            3, "Current Level");

    checkErr(rsngx_QueryViReal64(instrSession, "VOLT? MAX", maxVoltageLevel));

Error:
    if (old_current_level > 0.0)
    {
        checkErr(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_LEVEL, old_current_level));
    }
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the current value of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR currentLevel/Sets the current value of the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureCurrentLevel(ViSession instrSession,
                                              ViInt32   channel,
                                              ViReal64  currentLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_LEVEL, currentLevel),
            3, "Current Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the range for target current.
/// HIFN
/// HIFN     Refer the datasheet for the measurement accuracy according to the
/// HIFN  measurement range selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR range/Sets or queries the range for target current.
ViStatus _VI_FUNC rsngx_ConfigureCurrentRange(ViSession instrSession,
                                              ViInt32   channel,
                                              ViReal64  range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_CURRENT_RANGE, range),
            3, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the lower safety limit for current.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR lowerSafetyLimit/Sets the lower safety limit for current.
/// HIPAR upperSafetyLimit/Sets the upper safety limit for current.
ViStatus _VI_FUNC rsngx_ConfigureCurrentSafetyLimit(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViReal64  lowerSafetyLimit,
                                                    ViReal64  upperSafetyLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_SAFETY_LIMIT_LOWER, lowerSafetyLimit),
            3, "Lower Safety Limit");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CURRENT_SAFETY_LIMIT_UPPER, upperSafetyLimit),
            4, "Upper Safety Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method returns the maximum programmable current limit that the
/// HIFN  power supply accepts for a particular voltage level on an output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltageLevel/The voltage level for which to determine the maximum programmable
/// HIPAR voltageLevel/current limit.
/// HIPAR maxCurrentLevel/Returns the maximum programmable current level for the specified
/// HIPAR maxCurrentLevel/voltage level.
ViStatus _VI_FUNC rsngx_QueryMaxCurrentLevel(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  voltageLevel,
                                             ViReal64* maxCurrentLevel)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 old_voltage_level = 0.0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_LEVEL, &old_voltage_level));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_LEVEL, voltageLevel),
            3, "Current Level");

    checkErr(rsngx_QueryViReal64(instrSession, "CURR? MAX", maxCurrentLevel));

Error:
    if (old_voltage_level > 0.0)
    {
        checkErr(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_VOLTAGE_LEVEL, old_voltage_level));
    }
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the constant resistance mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the constant resistance mode.
/// HIPAR level/Sets the constant resistance target value.
ViStatus _VI_FUNC rsngx_ConfigureResistance(ViSession instrSession,
                                            ViInt32   channel,
                                            ViBoolean enabled,
                                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_RESISTANCE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_RESISTANCE_LEVEL, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches ON/OFF the output of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Switches ON/OFF the output of the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureOutputEnabled(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureOutputSelectedChannelEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_SELECTED_CHANNEL_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches ON/OFF all previous selected channels simultaneously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Switches ON/OFF all previous selected channels simultaneously.
ViStatus _VI_FUNC rsngx_ConfigureOutputAllEnabled(ViSession instrSession,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_ALL_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the output delay for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the output delay for the selected channel.
/// HIPAR duration/Sets the duration for output delay.
ViStatus _VI_FUNC rsngx_ConfigureOutputDelay(ViSession instrSession,
                                             ViInt32   channel,
                                             ViBoolean enabled,
                                             ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_DELAY_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OUTPUT_DELAY_DURATION, duration),
            4, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the fast transient response.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the fast transient response.
ViStatus _VI_FUNC rsngx_ConfigureOutputFastTransientResponseEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_FAST_TRANSIENT_RESPONSE_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the reduce slew rate option for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR reducedSlewRate/Sets or queries the reduce slew rate option for the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureOutputReducedSlewRateEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean reducedSlewRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_OUTPUT_REDUCED_SLEW_RATE_ENABLED, reducedSlewRate),
            3, "Reduced Slew Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  Outputs source impedance for the signal specified in ohms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the impedance target for the selected channel.
/// HIPAR impedance/Outputs source impedance for the signal specified in ohms.
ViStatus _VI_FUNC rsngx_ConfigureOutputImpedance(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean enabled,
                                                 ViReal64  impedance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_IMPEDANCE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OUTPUT_IMPEDANCE, impedance),
            4, "Impedance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the output mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR outputMode/Sets the output mode.
ViStatus _VI_FUNC rsngx_ConfigureOutputMode(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   outputMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_OUTPUT_MODE, outputMode),
            3, "Output Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for triggered event for output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR triggerEnabled/Enables or disables the triggered event for output.
/// HIPAR triggeredBehavior/Sets or queries output behavior when a trigger event occurs.
ViStatus _VI_FUNC rsngx_ConfigureOutputTriggerSettings(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViBoolean triggerEnabled,
                                                       ViInt32   triggeredBehavior)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OUTPUT_TRIGGER_ENABLED, triggerEnabled),
            3, "Trigger Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "",
                    RSNGX_ATTR_OUTPUT_TRIGGERED_BEHAVIOR, triggeredBehavior), 4, "Triggered Behavior")

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets remote sense detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR remoteSenseDetection/Sets remote sense detection.
ViStatus _VI_FUNC rsngx_ConfigureOutputRemoteSenseDetection(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   remoteSenseDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_OUTPUT_REMOTE_SENSE_DETECTION, remoteSenseDetection),
            2, "Remote Sense Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries remote sense detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR remoteSenseDetection/Queries remote sense detection.
ViStatus _VI_FUNC rsngx_QueryOutputRemoteSenseDetection(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32*  remoteSenseDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_OUTPUT_QUERY_REMOTE_SENSE_DETECTION, remoteSenseDetection),
            3, "Remote Sense Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the output capacitance mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR capacitanceMode/Sets or queries the output capacitance mode.
ViStatus _VI_FUNC rsngx_ConfigureOutputCapacitanceMode(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   capacitanceMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_OUTPUT_CAPACITANCE_MODE, capacitanceMode),
            3, "Capacitance Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for DMV measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enableDVM/Enables or disables digital voltmeter measurements.
/// HIPAR enableVoltageAutorange/Enables or disables auto range for voltage measurement accuracy.
/// HIPAR selectVoltageRange/Sets the voltage range for measurement. There is a selection of 20 V
/// HIPAR selectVoltageRange/and 5 V range.
/// HIPAR enableCurrentAutorange/Enables or disables auto range for current measurement accuracy.
/// HIPAR selectCurrentRange/Sets the current range for measurement. There is a selection of 10 A,
/// HIPAR selectCurrentRange/1 A, 100 mA and 10 mA range.
ViStatus _VI_FUNC rsngx_ConfigureDVMMeasurementSettings(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean enableDVM,
                                                        ViBoolean enableVoltageAutorange,
                                                        ViInt32   selectVoltageRange,
                                                        ViBoolean enableCurrentAutorange,
                                                        ViInt32   selectCurrentRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_DVM_MEASUREMENT_ENABLED, enableDVM),
            3, "Enable DVM");

    if (enableDVM == VI_FALSE)
    {
        viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_VOLTAGE_AUTORANGE_ENABLED, enableVoltageAutorange),
                4, "Enable Voltage Autorange");

        if (enableVoltageAutorange == VI_FALSE)
        {
            viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_VOLTAGE_RANGE, selectVoltageRange),
                    5, "Select Voltage Range");
        }
    }

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_CURRENT_AUTORANGE_ENABLED, enableCurrentAutorange),
            6, "Enable Current Autorange");

    if (enableCurrentAutorange == VI_FALSE)
    {
        viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CURRENT_RANGE, selectCurrentRange),
                7, "Select Current Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the smallest range that instrument autorange switch
/// HIFN  to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR autorangeMinimum/Sets or queries the smallest range that instrument autorange switch
/// HIPAR autorangeMinimum/to.
ViStatus _VI_FUNC rsngx_ConfigureCurrentAutorangeMinimum(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViReal64  autorangeMinimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_CURRENT_AUTORANGE_MINIMUM, autorangeMinimum),
            3, "Autorange Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries how autorange handles an overrange condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR approach/Sets or queries how autorange handles an overrange condition.
ViStatus _VI_FUNC rsngx_ConfigureCurrentAutorangeOverApproach(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   approach)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_CURRENT_AUTORANGE_OVER_APPROACH, approach),
            3, "Approach");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the threshold durations before and after the
/// HIFN  instrument switches the measurement range when an overrange condition
/// HIFN  is detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR upperThreshold/Sets or queries the threshold duration before the instrument switches
/// HIPAR upperThreshold/the measurement range when an overrange condition is detected.
/// HIPAR lowerThreshold/Sets or queries the threshold duration for autorange in the underrange
/// HIPAR lowerThreshold/condition.
ViStatus _VI_FUNC rsngx_ConfigureCurrentAutorangeThresholds(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViReal64  upperThreshold,
                                                            ViReal64  lowerThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_CURRENT_AUTORANGE_UPPER_THRESHOLD, upperThreshold),
            3, "Upper Threshold");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_CURRENT_AUTORANGE_LOWER_THRESHOLD, lowerThreshold),
            4, "Lower Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the fuse function in the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Sets the state for over current protection (OCP).
/// HIPAR delay/Sets a delay time for the fuse to take effect.
ViStatus _VI_FUNC rsngx_ConfigureOCP(ViSession instrSession,
                                     ViInt32   channel,
                                     ViBoolean enabled,
                                     ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OCP_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OCP_DELAY, delay),
            4, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an initial delay time for the fuse to take effect.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR initialDelay/Sets the initial fuse delay time once output turns on.
ViStatus _VI_FUNC rsngx_ConfigureOCPInitialDelay(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViReal64  initialDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OCP_INITIAL_DELAY, initialDelay),
            3, "Initial Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures linking and unlinking of fuses from the other channel (Ch1
/// HIFN  or Ch2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR ocpLinkingType/The kind of linking to be configured.
/// HIPAR ocpLink/Sets the fuses of several selected channels (fuse linking).
ViStatus _VI_FUNC rsngx_ConfigureOCPLinking(ViSession instrSession,
                                            ViInt32   channel,
                                            ViInt32   OCPLinkingType,
                                            ViInt32   OCPLink)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSNGX_ATTR_OCP_LINK, RSNGX_ATTR_OCP_UNLINK};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, OCPLinkingType, RSNGX_VAL_FUSE_LINKING, RSNGX_VAL_FUSE_UNLINKING),
            3, "OCP Linking Type");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    if (channel != OCPLink)
    {
        viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", attr[OCPLinkingType], OCPLink),
                4, "OCP Link");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries if channels are linked (fuse linking). Returns the link
/// HIRET Returns the link status between the currently selected channel and the parameter.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR Number of channel that queried OCP link is linked to.
/// HIPAR linkState/Returns the link status between the currently selected channel and the parameter (linked channel).
ViStatus _VI_FUNC rsngx_QueryOCPLinking(ViSession  instrSession,
                                        ViInt32    channel,
                                        ViInt32    linkedChannel,
                                        ViBoolean* linkState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CURR:PROT:LINK? %ld", linkedChannel);
    checkErr(rsngx_QueryViBoolean(instrSession, cmd, linkState));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the status if fuse has tripped in the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR fuseTripped/Queries the status if fuse has tripped in the selected channel.
ViStatus _VI_FUNC rsngx_QueryOCPTripped(ViSession  instrSession,
                                        ViInt32    channel,
                                        ViBoolean* ocpTripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OCP_TRIPPED, ocpTripped),
            3, "OCP Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the OCP state of the selected channel. If an OCP event has
/// HIFN  occurred before, the reset also erases the message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetOCP(ViSession instrSession,
                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_OCP_RESET, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures settings for over current protection (OCP).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Sets or queries the state for over current protection (OCP).
/// HIPAR delayTime/Sets the fuse delay time.
ViStatus _VI_FUNC rsngx_ConfigureOCPFuse(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   enabled,
                                         ViReal64  delayTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_OCP_FUSE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OCP_FUSE_DELAY_TIME, delayTime),
            4, "Delay Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the initial fuse delay time once output turns on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR initialDelay/Sets the initial fuse delay time once output turns on.
ViStatus _VI_FUNC rsngx_ConfigureOCPFuseInitialDelay(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViReal64  initialDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OCP_FUSE_DELAY_AT_OUTPUT_ON, initialDelay),
            3, "Initial Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures linking and unlinking of fuses to a selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR fuseLinkingType/The kind of linking to be configured.
/// HIPAR fuseLink/Sets or queries the fuses of several selected channels (fuse linking).
ViStatus _VI_FUNC rsngx_ConfigureOCPFuseLinking(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   fuseLinkingType,
                                                ViInt32   fuseLink)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    if (fuseLinkingType == RSNGX_VAL_FUSE_LINKING)
    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_OCP_FUSE_LINK, fuseLink),
            4, "Fuse Link");

    if (fuseLinkingType == RSNGX_VAL_FUSE_UNLINKING)
    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_OCP_FUSE_UNLINK, fuseLink),
            4, "Fuse Link");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries if channels are linked (fuse linking). Returns a list of fuses
/// HIFN  linked to the currently selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/"Linked Fuses List" control.
/// HIPAR linkedFusesList/Queries if channels are linked (fuse linking). Returns a list of fuses
/// HIPAR linkedFusesList/linked to the currently selected channel.
ViStatus _VI_FUNC rsngx_QueryOCPLinkedFusesList(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   bufferSize,
                                                ViChar    linkedFusesList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_OCP_LINKED_FUSES_LIST,
            bufferSize, linkedFusesList),
            3, "Linked Fuses List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the OCP state of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR ocpTripped/Queries the OCP state of the selected channel.
ViStatus _VI_FUNC rsngx_QueryOCPFuseTripped(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* ocpTripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OCP_FUSE_IS_TRIPPED, ocpTripped),
            3, "OCP Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the OCP state of the selected channel. If an OCP event has
/// HIFN  occurred before, the reset also erases the message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetOCPFuse(ViSession instrSession,
                                     ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_OCP_FUSE_RESET, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the OVP (over voltage protection) function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Sets the OVP state of the previous selected channel.
/// HIPAR level/Sets the overvoltage protection value of the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureOVP(ViSession instrSession,
                                     ViInt32   channel,
                                     ViBoolean enabled,
                                     ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OVP_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OVP_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the OVP state of the selected channel. If an OVP event has
/// HIFN  occurred before, the reset also erases the message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetOVP(ViSession instrSession,
                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_OVP_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the OVP state of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR tripped/Queries the OVP state of the selected channel.
ViStatus _VI_FUNC rsngx_QueryOVPTripped(ViSession  instrSession,
                                        ViInt32    channel,
                                        ViBoolean* tripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OVP_TRIPPED, tripped),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables and configures the OPP (over power protection) function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Sets the OPP state of the previous selected channel.
/// HIPAR level/Sets the overvoltage protection value of the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureOPP(ViSession instrSession,
                                     ViInt32   channel,
                                     ViBoolean enabled,
                                     ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OPP_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_OPP_LEVEL, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the OPP state of the selected channel. If an OPP event has
/// HIFN  occurred before, the reset also erases the message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetOPP(ViSession instrSession,
                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_OPP_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the OPP state of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR tripped/Queries the OPP state of the selected channel.
ViStatus _VI_FUNC rsngx_QueryOPPTripped(ViSession  instrSession,
                                        ViInt32    channel,
                                        ViBoolean* tripped)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, "", RSNGX_ATTR_OPP_TRIPPED, tripped),
            3, "Tripped");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reset protection tripped state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ResetProtectionTrippedState(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_RESET_PROTECTION_TRIPPED_STATE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the remote sense tripped state of the selected channel. If the
/// HIFN  remote sense tripped event has occurred before, the reset also erases
/// HIFN  the message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetVoltageTrippedState(ViSession instrSession,
                                                 ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_RESET_VOLTAGE_TRIPPED_STATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the remote sense tripped state of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR trippedState/Queries the remote sense tripped state of the selected channel.
ViStatus _VI_FUNC rsngx_QueryVoltageTrippedState(ViSession  instrSession,
                                                 ViInt32    channel,
                                                 ViBoolean* trippedState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_VOLTAGE_TRIPPED_STATE, trippedState),
            3, "Tripped State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables and configures the EasyRamp function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the Ramp function for the previous selected
/// HIPAR enabled/channel.
/// HIPAR duration/Sets the duration of the voltage ramp.
ViStatus _VI_FUNC rsngx_ConfigureRamp(ViSession instrSession,
                                      ViInt32   channel,
                                      ViBoolean enabled,
                                      ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_RAMP_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_RAMP_DURATION, duration),
            4, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the source priority mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR priorityMode/Sets the source priority mode.
ViStatus _VI_FUNC rsngx_ConfigureSourcePriorityMode(ViSession instrSession,
                                                    ViInt32   priorityMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SOURCE_PRIORITY_MODE, priorityMode),
            2, "Priority Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of power line cycles for measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerLineCycles/Sets the number of power line cycles for measurements.
ViStatus _VI_FUNC rsngx_ConfigureNumberOfPowerLineCycles(ViSession instrSession,
                                                         ViInt32   powerLineCycles)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_NUMBER_OF_POWER_LINE_CYCLES, powerLineCycles),
            2, "Power Line Cycles");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enable/disable modulation of DVM input onto the output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR modulation/Enable/disable modulation of DVM input onto the output signal.
ViStatus _VI_FUNC rsngx_ConfigureModulationEnabled(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_MODULATION_ENABLED, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of power line cycles for measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gain/Sets the modulation gain.
/// HIPAR gainValue/Sets the modulation gain.
ViStatus _VI_FUNC rsngx_ConfigureModulationGain(ViSession instrSession,
                                                ViInt32   gain,
                                                ViReal64  gainValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_MODULATION_GAIN, gain),
            2, "Gain");

    if (gain == RSNGX_VAL_VOLTAGE_CURRENT_NEGATIVE_VAL)
    {
        viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_MODULATION_GAIN_VALUE, gainValue),
                3, "Gain Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the status of the master tracking state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingEnabled/Sets or queries the status of the master tracking state.
ViStatus _VI_FUNC rsngx_ConfigureTrackingEnabled(ViSession instrSession,
                                                 ViInt32   trackingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_TRACKING_ENABLED, trackingEnabled),
            2, "Tracking Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the tracking status on selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR trackingEnabled/Sets or queries the tracking status on selected channel.
ViStatus _VI_FUNC rsngx_ConfigureTrackingChannelEnabled(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   trackingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_TRACKING_CHANNEL_ENABLED,
            "Channel", channel -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_TRACKING_CHANNEL_ENABLED, trackingEnabled),
            3, "Tracking Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the status of tracking soft enable on specific
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR trackingSoftEnabled/Sets or queries the status of tracking soft enable on specific
/// HIPAR trackingSoftEnabled/channel.
ViStatus _VI_FUNC rsngx_ConfigureTrackingChannelSoftEnabled(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViInt32   trackingSoftEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_TRACKING_CHANNEL_SOFT_ENABLED,
            "Channel", channel -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_TRACKING_CHANNEL_SOFT_ENABLED, trackingSoftEnabled),
            3, "Tracking Soft Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Takes a measurement on the output signal and returns the measured
/// HIFN  value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR measurementType/The type of measurement to take.
/// HIPAR data/The measured value.
ViStatus _VI_FUNC rsngx_QueryMeasurementData(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   measurementType,
                                             ViReal64* data)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSNGX_ATTR_MEAS_VOLTAGE,
             RSNGX_ATTR_MEAS_CURRENT,
             RSNGX_ATTR_MEAS_POWER,
             RSNGX_ATTR_MEAS_ENERGY
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSNGX_VAL_MEASURE_VOLTAGE, RSNGX_VAL_MEASURE_ENERGY),
            3, "Measurement Type");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", attr[measurementType], data),
            4, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the voltmeter measurement (if DVM is enabled).
/// HIFN     The DVM is available only with R&S NGM power supply series equipped
/// HIFN  with R&S NGM-K104 (P/N: 3643.9927.02).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR data/Queries the voltmeter measurement (if DVM is enabled).
ViStatus _VI_FUNC rsngx_QueryMeasurementDataDVM(ViSession instrSession,
                                                ViReal64* data)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_MEAS_VOLTAGE_DVM, data),
            2, "Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the energy counter for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the energy counter for the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureEnergyCounterEnabled(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_MEAS_ENERGY_COUNTER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the energy counter for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetEnergyCounter(ViSession instrSession,
                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_MEAS_ENERGY_COUNTER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the measured unit for energy.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Sets or queries the measured unit for energy.
ViStatus _VI_FUNC rsngx_ConfigureEnergyCounterUnit(ViSession instrSession,
                                                   ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ENERGY_COUNTER_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Takes a measurement on the output signal and returns the measured
/// HIFN  statistical value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR measurementType/The type of measurement to take.
/// HIPAR statisticsType/Selects the statistical type.
ViStatus _VI_FUNC rsngx_QueryMeasurementStatisticsData(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   measurementType,
                                                       ViInt32   statisticsType,
                                                       ViReal64* statisticsData)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   avgMeasurementAttr[3][3] = {
             {
             RSNGX_ATTR_MEASUREMENT_VOLTAGE_AVG,
             RSNGX_ATTR_MEASUREMENT_CURRENT_AVG,
             RSNGX_ATTR_MEASUREMENT_POWER_AVG
             },
             {
             RSNGX_ATTR_MEASUREMENT_VOLTAGE_MIN,
             RSNGX_ATTR_MEASUREMENT_CURRENT_MIN,
             RSNGX_ATTR_MEASUREMENT_POWER_MIN
             },
             {
             RSNGX_ATTR_MEASUREMENT_VOLTAGE_MAX,
             RSNGX_ATTR_MEASUREMENT_CURRENT_MAX,
             RSNGX_ATTR_MEASUREMENT_POWER_MAX
             }
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSNGX_VAL_MEASURE_VOLTAGE, RSNGX_VAL_MEASURE_POWER),
            3, "Measurement Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticsType, RSNGX_VAL_AVG, RSNGX_VAL_MAX),
            4, "Statitstics Type");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", avgMeasurementAttr[statisticsType][measurementType], statisticsData),
            5, "Statictics Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current statistics of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR measurementType/The type of measurement to take.
/// HIPAR statisticsData/Queries the current statistics of the selected channel.
ViStatus _VI_FUNC rsngx_QueryMeasurementStatisticsDataAll(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViInt32   measurementType,
                                                          ViInt32   arraySize,
                                                          ViReal64  statisticsData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s:STAT", statDataArr[measurementType]);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &statisticsData, &arraySize));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the minimum, maximum and average statistic values for voltage,
/// HIFN  current, and power. Additionally this command resets the measured
/// HIFN  energy.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetMeasurementStatistics(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_MEASUREMENT_RESET_STATISTICS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR measurementCount/Queries the number of measurements.
ViStatus _VI_FUNC rsngx_QueryMeasurementCount(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32*  measurementCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_MEASUREMENT_COUNT, measurementCount),
            3, "Measurement Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the global data logging function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets/Queries the data logging function on or off.
/// HIPAR mode/Sets/Queries the data logging mode.
/// HIPAR count/Sets the number of measurement values to be captured.
/// HIPAR duration/Sets the duration of the data logging.
/// HIPAR interval/Sets the data logging measurement interval. The measurement interval
/// HIPAR interval/describes the time between the recorded measurements.
ViStatus _VI_FUNC rsngx_ConfigureLogging(ViSession instrSession,
                                         ViBoolean enabled,
                                         ViInt32   mode,
                                         ViInt32   count,
                                         ViReal64  duration,
                                         ViReal64  interval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_MEAS_LOG_MODE, mode),
            3, "Mode");

    switch (mode)
    {
    case RSNGX_VAL_LOGGING_MODE_UNLIMITED:
        {
            break;
        }
    case RSNGX_VAL_LOGGING_MODE_COUNT:
        {
            viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_MEAS_LOG_COUNT, count),
                    4, "Count");

            break;
        }
    case RSNGX_VAL_LOGGING_MODE_DURATION:
        {
            viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_MEAS_LOG_DURATION, duration),
                    5, "Duration");

            break;
        }
    case RSNGX_VAL_LOGGING_MODE_SPAN:
        {
            viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_MEAS_LOG_INTERVAL, interval),
                    6, "Interval");
            break;
        }
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_MEAS_LOG_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the start time of the data logging fucntion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Sets the year for the data logging function.
/// HIPAR month/Sets the month for the data logging function.
/// HIPAR day/Sets the day for the data logging function.
/// HIPAR hour/Sets the hour for the data logging function.
/// HIPAR minute/Sets the minute for the data logging function.
/// HIPAR second/Sets the second for the data logging function.
ViStatus _VI_FUNC rsngx_ConfigureLoggingStartTime(ViSession instrSession,
                                                  ViInt32   year,
                                                  ViInt32   month,
                                                  ViInt32   day,
                                                  ViInt32   hour,
                                                  ViInt32   minute,
                                                  ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LOG:STIM %ld,%02ld,%02ld,%02ld,%02ld,%02ld",
            year, month, day, hour, minute, second);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the manual trigger for the data logging function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggered/Sets the manual trigger for the data logging function.
ViStatus _VI_FUNC rsngx_ConfigureLoggingTriggered(ViSession instrSession,
                                                  ViBoolean triggered)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_MEAS_LOG_TRIGGERED, triggered),
            2, "Triggered");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the logging location.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR location/Sets or queries the logging location.
ViStatus _VI_FUNC rsngx_ConfigureLogLocation(ViSession instrSession,
                                             ViInt32   location)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_LOG_LOCATION, location),
            2, "Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns 12 sets of latest logging data with minimum logging interval
/// HIFN  (8 ms).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arraySize/Determines the array size for the data log data.
/// HIPAR logData/Returns 12 sets of latest logging data with minimum logging interval
/// HIPAR logData/(8 ms).
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsngx_ReadLogData(ViSession instrSession,
                                    ViInt32   channel,
                                    ViInt32   arraySize,
                                    ViReal64  logData[],
                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL, RSNGX_ATTR_HIDDEN_LOG_DATA, NULL, arraySize, logData, returnedValues),
        4, "Log Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the filename and storage location for the logging function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Filename of the data logging function.
/// HIPAR location/Parameter is currently unused.
ViStatus _VI_FUNC rsngx_ConfigureLoggingFile(ViSession instrSession,
                                             ViString  filename,
                                             ViInt32   location)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             location;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LOG:FNAM '%s'", filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the logging file data of the selected file. If manual trigger
/// HIFN  mode (trigger via TRIG function) is used, the logging function has to
/// HIFN  be activated. Without activating the logging function in the manual
/// HIFN  trigger mode, the instrument is not able to save a logging file
/// HIFN  internally or on the USB stick.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the "File
/// HIPAR bufferSize/Content" control.
/// HIPAR fileContent/Read the file "Filename" and returns the file content.
/// HIPAR responseLength/Returns the file content length.
ViStatus _VI_FUNC rsngx_ReadFileFromInstrument(ViSession instrSession,
                                               ViString  filename,
                                               ViInt32   bufferSize,
                                               ViChar    fileContent[],
                                               ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DATA? '%s'", filename);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, fileContent, responseLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the specified file from memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileToDelete/Deletes the specified file from memory.
ViStatus _VI_FUNC rsngx_DeleteFile(ViSession instrSession,
                                   ViString  fileToDelete)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_MEAS_LOG_DELETE, fileToDelete),
            2, "Delete");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of measurements from the selected logging file. If
/// HIFN  manual trigger mode (trigger via TRIG function) is used, the logging
/// HIFN  function has to be activated. Without activating the logging function
/// HIFN  in the manual trigger mode, the instrument is not able to save a
/// HIFN  logging file internally or on the USB stick.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Filename of the data logging function.
/// HIPAR points/Returns the number of measurements.
ViStatus _VI_FUNC rsngx_ReadDataPoints(ViSession instrSession,
                                       ViString  filename,
                                       ViUInt32* points)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:POIN? '%s'", filename);
    checkErr(rsngx_QueryViInt32(instrSession, cmd, (ViInt32*)points));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all files in internal memory ('/int/') and external memory
/// HIFN  ('/USB').
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/"Catalog" control.
/// HIPAR catalog/Queries all files in internal memory ('/int/') and external memory
/// HIPAR catalog/('/USB').
ViStatus _VI_FUNC rsngx_ReadDataCatalog(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_MEAS_LOG_READ_DATA_CATALOG,
            bufferSize, catalog),
            3, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the FastLog state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Sets the FastLog state.
ViStatus _VI_FUNC rsngx_ConfigureFastLogEnabled(ViSession instrSession,
                                                ViInt32   channel,
                                                ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_channelRepCap(instrSession, channel, repCap));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, repCap, RSNGX_ATTR_FAST_LOG_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for Fast Logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR writetoFile/Sets the FastLog data to be written to an external file.
/// HIPAR writeDuration/Sets the file writes duration.
/// HIPAR sampleRate/Sets the sample rate of the FastLog function.
ViStatus _VI_FUNC rsngx_ConfigureFastLogSettings(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   writeTo,
                                                 ViReal64  writeDuration,
                                                 ViInt32   sampleRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_channelRepCap(instrSession, channel, repCap));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_FAST_LOG_WRITE_TO, writeTo),
            3, "Write To");

    if (writeTo == RSNGX_VAL_FAST_LOG_WRITE_TO_USB)
    {
        viCheckParm(rsngx_SetAttributeViReal64(instrSession, repCap, RSNGX_ATTR_FAST_LOG_WRITE_DURATION, writeDuration),
                4, "Write Duration");
    }

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_FAST_LOG_SAMPLE_RATE, sampleRate),
            5, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the triggered state of FastLog.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR triggered/Sets the triggered state of FastLog.
ViStatus _VI_FUNC rsngx_ConfigureFastLogTriggered(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean triggered)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_channelRepCap(instrSession, channel, repCap));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, repCap, RSNGX_ATTR_FAST_LOG_TRIGGERED, triggered),
            3, "Triggered");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries FastLog data into a binary file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR responseLength/Returns the file content length.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the "File
/// HIPAR bufferSize/Content" control.
/// HIPAR fileContent/Read the file "Filename" and returns the file content.
ViStatus _VI_FUNC rsngx_ReadFastLogDataFromInstrument(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32*  responseLength,
                                                      ViInt32   bufferSize,
                                                      ViChar    fileContent[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FLOG%ld:DATA?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, fileContent, responseLength));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the external partition to which the data is written into.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR fastLogPartition/Selects the external partition to which the data is written into.
ViStatus _VI_FUNC rsngx_ConfigureFastLogPartition(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViString  fastLogPartition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_channelRepCap(instrSession, channel, repCap));

    viCheckParm(rsngx_SetAttributeViString(instrSession, repCap, RSNGX_ATTR_FAST_LOG_PARTITION, fastLogPartition),
            3, "Fast Log Partition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the samples accumulate duration in seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR sampleTime/Sets or queries the samples accumulate duration in seconds.
ViStatus _VI_FUNC rsngx_ConfigureFastLogSampleTime(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViReal64  sampleTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_FAST_LOG_SAMPLE_TIME, sampleTime),
            3, "Sample Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the FastLog settings for data to be written to an external file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR externalFile/Sets the FastLog data to be written to an external file.
/// HIPAR duration/Sets or queries the file writes duration.
/// HIPAR partition/Selects the external partition to which the data is written into.
ViStatus _VI_FUNC rsngx_ConfigureFastLogExternalFileSettings(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViBoolean externalFile,
                                                             ViReal64  duration,
                                                             ViString  partition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_FAST_LOG_EXTERNAL_FILE_ENABLED, externalFile),
            3, "External File");

    if (externalFile == VI_TRUE)
    {
        viCheckParm(rsngx_SetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_FAST_LOG_EXTERNAL_FILE_DURATION, duration),
                4, "Duration");

        viCheckParm(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_FAST_LOG_EXTERNAL_FILE_PARTITION, partition),
                5, "Partition");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Choose the target the data shall be written to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR target/Choose the target the data shall be written to.
ViStatus _VI_FUNC rsngx_ConfigureFastLogTarget(ViSession instrSession,
                                               ViInt32   target)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_FAST_LOG_TARGET, target),
            2, "Target");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the arbitrary function of the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates or Queries the arbitrary function for the
/// HIPAR enabled/previous selected channel.
/// HIPAR repetitionRate/Sets the repetition rate of the defined arbitrary waveform for the
/// HIPAR repetitionRate/previous selected channel. Up to 255 repetitions are possible. If the
/// HIPAR repetitionRate/repetition rate 0 is selected the arbitrary waveform of the previous
/// HIPAR repetitionRate/selected channel will be repeated infinitely.
/// HIPAR endpoint/Not used
/// HIPAR endpointBehavior/Sets the arbitrary endpoint behavior, when arbitrary function is
/// HIPAR endpointBehavior/finished.
ViStatus _VI_FUNC rsngx_ConfigureArbitrary(ViSession instrSession,
                                           ViInt32   channel,
                                           ViBoolean enabled,
                                           ViInt32   repetitionRate,
                                           ViInt32   endpoint,
                                           ViInt32   endpointBehavior)
{
    ViStatus error = VI_SUCCESS;
             endpoint;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_ARBITRARY_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_REPETITION_RATE, repetitionRate),
            4, "Repetition Rate");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_ENDPOINT_BEHAVIOR, endpointBehavior),
            6, "Endpoint Behavior");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the filename, device and operation for arbitrary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR filename/Filename of the arbitrary function.
/// HIPAR location/Arbitrary function location.
/// HIPAR operation/Selects the file operation.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryFile(ViSession instrSession,
                                               ViInt32   channel,
                                               ViString  filename,
                                               ViInt32   location,
                                               ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAttr   attr[] = {RSNGX_ATTR_ARBITRARY_SAVE_TABLE, RSNGX_ATTR_ARBITRARY_LOAD_TABLE};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, location, RSNGX_VAL_LOCATION_INT, RSNGX_VAL_LOCATION_DEF),
            4, "Location");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSNGX_VAL_FILE_OPER_SAVE, RSNGX_VAL_FILE_OPER_LOAD),
            5, "Operation");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngLocation, location, &p2string),
                4, "Location");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ARB:FNAM '%s',%s", filename, p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", attr[operation], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the previous defined arbitrary waveform data for the selected
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetArbitrary(ViSession instrSession,
                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ARBITRARY_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Transfers the defined arbitrary table to the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR transferTableToChannel/Transfers the defined arbitrary table to the selected channel.
ViStatus _VI_FUNC rsngx_TransferArbitraryTable(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   transferTableToChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_TRANSFER_TABLE, transferTableToChannel),
            3, "Transfer Table To Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the trigger input at the rear connector
/// HIFN  (terminal block) for arbitrary functionality.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Activates/Deactivates the trigger input at the rear connector
/// HIPAR enabled/(terminal block) for arbitrary functionality.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryTriggerInputEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_ARBITRARY_TRIGGER_INPUT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the trigger condition of the arbitrary step group for
/// HIFN  the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR groupChannel/Sets or queries the trigger condition of the arbitrary step group for
/// HIPAR groupChannel/the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryTriggerGroupChannel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   groupChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_TRIGGER_GROUP_CHANNEL, groupChannel),
            3, "Group Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the trigger condition of the arbitrary step point for
/// HIFN  the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR pointChannel/Sets or queries the trigger condition of the arbitrary step point for
/// HIPAR pointChannel/the selected channel.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryTriggerPointChannel(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   pointChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_TRIGGER_POINT_CHANNEL, pointChannel),
            3, "Point Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the arbitrary trigger mode of the previous selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arbitraryTriggerMode/Sets the arbitrary trigger mode of the previous selected channel.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryTriggerMode(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   arbitraryTriggerMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_TRIGGER_MODE, arbitraryTriggerMode),
            3, "Arbitrary Trigger Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the arbitrary points for the previous selected channel. Max. 512
/// HIFN  arbitrary points can be defined. The dwell time between 2 arbitrary
/// HIFN  points is specified from 10 ms to 10 minutes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR numberofvalues/The number of values written.
/// HIPAR voltage/Writes an array of voltage.
/// HIPAR current/Writes an array of current.
/// HIPAR time/Writes an array of time.
/// HIPAR interpolationMode/Writes an array of interpolation mode.
ViStatus _VI_FUNC rsngx_WriteArbitraryData(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   numberofvalues,
                                           ViReal64  voltage[],
                                           ViReal64  current[],
                                           ViReal64  time[],
                                           ViInt32   interpolationMode[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    if (numberofvalues > 0)
    {
        p2buf += sprintf(p2buf, "ARB:DATA ");

        for (i = 0; i < numberofvalues; i++)
        {
            p2buf += sprintf(p2buf, "%.12lG,%.12lG,%.12lG,%ld", voltage[i], current[i], time[i], interpolationMode[i]);
            if (i < numberofvalues - 1)
                *(p2buf++) = ',';
        }

        *p2buf = '\0';
        checkErr(RsCore_Write(instrSession, cmd));
    }
    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the arbitrary points for the previous selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arraySize/This parameter determines the size of array Voltage, Current, Time and
/// HIPAR arraySize/Interpolation Mode.
/// HIPAR voltage/Returns an array of voltage.
/// HIPAR current/Returns an array of current.
/// HIPAR time/Returns an array of time.
/// HIPAR interpolationMode/Returns an array of interpolation mode.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsngx_ReadArbitraryData(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   arraySize,
                                          ViReal64  voltage[],
                                          ViReal64  current[],
                                          ViReal64  time[],
                                          ViInt32   interpolationMode[],
                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  buffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  cnt = 1;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "ARB:DATA?", &buffer));

    p2buf = strtok(buffer, ",");

    while ((p2buf != NULL) && (i < arraySize))
    {
        voltage[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");
        current[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");
        time[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");
        interpolationMode[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");
        i++;
    }

    if (returnedValues != NULL)
    {
        while ((p2buf != NULL))
        {
            p2buf = strtok(NULL, ",");
            cnt++;
        }
        *returnedValues = i + (cnt + 1) / 4;
    }

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the arbitrary priority mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arbitraryPriorityMode/Sets or queries the arbitrary priority mode.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryPriorityMode(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   arbitraryPriorityMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_PRIORITY_MODE, arbitraryPriorityMode),
            3, "Arbitrary Priority Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the arbitrary points for the selected channel in current
/// HIFN  priority mode.
/// HIFN
/// HIFN     Note(s):
/// HIFN     Available only for NGU instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arraySize/Determines the array size for the arbitrary data.
/// HIPAR cpmData/Returns the arbitrary points for the selected channel in the current
/// HIPAR cpmData/priority mode.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsngx_ReadArbitraryCPMData(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   arraySize,
                                             ViReal64  cpmData[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, cpmData), 4, "CPM Data");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_PRIORITY_MODE, RSNGX_VAL_ARBITRARY_PRIORITY_MODE_CPM));

    checkErr(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL, RSNGX_ATTR_HIDDEN_ARBITRARY_DATA, NULL, arraySize, cpmData, returnedValues));

    if (*returnedValues % 5 != 0)
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected instrument response received (values count not a multiple of 5), make sure Arbitrary Priority Mode is set to CPM");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the arbitrary points for the selected channel in current priority
/// HIFN  mode. Max. 512 arbitrary points can be defined. The dwell time between
/// HIFN  2 arbitrary points is specified from 10 ms to 10 minutes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR numberofvalues/The number of values written.
/// HIPAR positiveVoltage/Writes an array of positive voltage values.
/// HIPAR current/Writes an array of current values.
/// HIPAR time/Writes an array of time values.
/// HIPAR interpolationMode/Writes an array of interpolation mode values.
/// HIPAR negativeVoltage/Writes an array of negative voltage values.
ViStatus _VI_FUNC rsngx_WriteArbitraryCPMData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   numberofvalues,
                                              ViReal64  positiveVoltage[],
                                              ViReal64  negativeVoltage[],
                                              ViReal64  current[],
                                              ViReal64  time[],
                                              ViInt32   interpolationMode[])
{
    ViStatus      error = VI_SUCCESS;
    ViChar*       param = NULL;
    ViConstString comma = ",";
    ViConstString delimiter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_PRIORITY_MODE, RSNGX_VAL_ARBITRARY_PRIORITY_MODE_CPM));

    for (int i = 0; i < numberofvalues; i++)
    {
        delimiter = i == 0 ? NULL : comma;
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, positiveVoltage[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, negativeVoltage[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, current[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, time[i], NULL));
        checkErr(RsCore_StrCatDynamicViInt32(instrSession, &param, delimiter, interpolationMode[i], NULL));
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSNGX_ATTR_HIDDEN_ARBITRARY_DATA, RS_VAL_DIRECT_USER_CALL, param));

Error:
    if (param) free(param);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the arbitrary points for the selected channel in voltage
/// HIFN  priority mode.
/// HIFN
/// HIFN     Note(s):
/// HIFN     Available only for NGU instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arraySize/Determines the array size for the arbitrary data.
/// HIPAR vpmData/Returns the arbitrary points for the selected channel in the voltage
/// HIPAR vpmData/priority mode.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsngx_ReadArbitraryVPMData(ViSession instrSession,
                                             ViInt32   channel,
                                             ViInt32   arraySize,
                                             ViReal64  vpmData[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, vpmData), 4, "VPM Data");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_PRIORITY_MODE, RSNGX_VAL_ARBITRARY_PRIORITY_MODE_VPM));

    checkErr(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL, RSNGX_ATTR_HIDDEN_ARBITRARY_DATA, NULL, arraySize, vpmData, returnedValues));

    if (*returnedValues % 5 != 0)
    {
        viCheckErrElab(RS_ERROR_UNEXPECTED_RESPONSE, "Unexpected instrument response received (values count not a multiple of 5), make sure Arbitrary Priority Mode is set to VPM");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the arbitrary points for the selected channel in voltage priority
/// HIFN  mode. Max. 512 arbitrary points can be defined. The dwell time between
/// HIFN  2 arbitrary points is specified from 10 ms to 10 minutes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR numberofvalues/The number of values written.
/// HIPAR voltage/Writes an array of voltage values.
/// HIPAR negativeCurrent/Writes an array of negative current values.
/// HIPAR time/Writes an array of time values.
/// HIPAR interpolationMode/Writes an array of interpolation mode values.
/// HIPAR positiveCurrent/Writes an array of positive current values.
ViStatus _VI_FUNC rsngx_WriteArbitraryVPMData(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   numberofvalues,
                                              ViReal64  voltage[],
                                              ViReal64  positiveCurrent[],
                                              ViReal64  negativeCurrent[],
                                              ViReal64  time[],
                                              ViInt32   interpolationMode[])
{
    ViStatus      error = VI_SUCCESS;
    ViChar*       param = NULL;
    ViConstString comma = ",";
    ViConstString delimiter;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_ARBITRARY_PRIORITY_MODE, RSNGX_VAL_ARBITRARY_PRIORITY_MODE_VPM));

    for (int i = 0; i < numberofvalues; i++)
    {
        delimiter = i == 0 ? NULL : comma;
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, voltage[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, positiveCurrent[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, negativeCurrent[i], NULL));
        checkErr(RsCore_StrCatDynamicViReal64(instrSession, &param, delimiter, time[i], NULL));
        checkErr(RsCore_StrCatDynamicViInt32(instrSession, &param, delimiter, interpolationMode[i], NULL));
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSNGX_ATTR_HIDDEN_ARBITRARY_DATA, RS_VAL_DIRECT_USER_CALL, param));

Error:
    if (param) free(param);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the arbitrary sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ClearArbitrarySequence(ViSession instrSession,
                                               ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_ARBITRARY_SEQUENCE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures arbitrary control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR control/Sets the control of the ARB.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryControl(ViSession instrSession,
                                                  ViInt32   control)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arrAttr[] = {
             RSNGX_ATTR_ARBITRARY_RESTORE,
             RSNGX_ATTR_ARBITRARY_START,
             RSNGX_ATTR_ARBITRARY_STOP
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, control, RSNGX_VAL_ARB_CONTROL_RESTORE, RSNGX_VAL_ARB_CONTROL_STOP),
            2, "Control");

    checkErr(rsngx_SetAttributeViString(instrSession, "", arrAttr[control], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the number of repetitions of the block of arbitrary
/// HIFN  data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
/// HIPAR repetitionRate/Sets or queries the number of repetitions of the block of arbitrary
/// HIPAR repetitionRate/data.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryBlockRepetitionRate(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   block,
                            ViInt32   repetitionRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_REPETITION_RATE, repetitionRate),
            4, "Repetition Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of data points of the block of arbitrary data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
/// HIPAR numberofDataPoints/Queries the number of data points of the block of arbitrary data.
ViStatus _VI_FUNC rsngx_QueryArbitraryBlockNumberOfDataPoints(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   block,
                            ViInt32*  numberofDataPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_NUMBER_OF_DATA_POINTS, numberofDataPoints),
            4, "Number of Data Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the file name and storage location for block of arbitrary data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
/// HIPAR filename/Filename of the arbitrary function.
/// HIPAR location/Arbitrary function location.
ViStatus _VI_FUNC rsngx_ConfigureArbitraryBlockFileName(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   block,
                                                        ViString  filename,
                                                        ViInt32   location)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngLocation, location, &p2string),
                4, "Location");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ARB:BLOC:FNAM '%s',%s", filename, p2string);
    checkErr(rsngx_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears a file selected for the block under channel arbitrary settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
ViStatus _VI_FUNC rsngx_ClearArbitraryBlockFile(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   block)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_CLEAR_FILE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the arbitrary points for the whole block.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
/// HIPAR arraySize/This parameter determines the size of array containing Arbitrary Data
/// HIPAR arraySize/in the block.
/// HIPAR voltage/Returns an array of voltage values in the Arbitrary Data.
/// HIPAR current/Returns an array of current values in the Arbitrary Data.
/// HIPAR time/Returns an array of time values in the Arbitrary Data.
/// HIPAR interpolationMode/Returns an array of interpolation mode statuses in the Arbitrary Data.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsngx_ReadBlockArbitraryData(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   block,
                                               ViInt32   arraySize,
                                               ViReal64  voltage[],
                                               ViReal64  current[],
                                               ViReal64  time[],
                                               ViInt32   interpolationMode[],
                                               ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* valuesRead = NULL;
    ViInt32   allValues = 0;
    ViInt32   i = 0;
    ViInt32   y = 0;
    ViInt32   count = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "ARBitrary:BLOCk:DATA?", &valuesRead, &allValues));

    checkErr(rsngx_CheckStatus(instrSession));

    count = (allValues / 3 > arraySize) ? arraySize : allValues / 3;

    while (count > 0)
    {
        voltage[i] = valuesRead[y++];
        current[i] = valuesRead[y++];
        time[i] = valuesRead[y++];
        interpolationMode[i] = (ViInt32)valuesRead[y++];
        i++;
        count--;
    }

    if (returnedValues) *returnedValues = i;

Error:
    if (valuesRead) free(valuesRead);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets arbitrary points for the whole block.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR block/Select individual block between 1 to 8 in an arbitrary sequence.
/// HIPAR numberofValues/The number of values written.
/// HIPAR voltage/An array of voltages written into arbitrary data.
/// HIPAR current/An array of curents written into arbitrary data.
/// HIPAR time/An array of times written into arbitrary data.
/// HIPAR interpolationMode/An array of interpolation modes written into arbitrary data.
ViStatus _VI_FUNC rsngx_WriteBlockArbitraryData(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   block,
                                                ViInt32   numberofValues,
                                                ViReal64  voltage [],
                                                ViReal64  current [],
                                                ViReal64  time [],
                                                ViInt32   interpolationMode [])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_BLOCK_SELECTED, block),
            3, "Block");

    if (numberofValues > 0)
    {
        p2buf += sprintf(p2buf, "ARB:BLOC:DATA ");

        for (int i = 0; i < numberofValues; i++)
        {
            p2buf += sprintf(p2buf, "%.12lG,%.12lG,%.12lG,%ld", voltage[i], current[i], time[i], interpolationMode[i]);
            if (i < numberofValues - 1)
                *(p2buf++) = ',';
        }

        *p2buf = '\0';

        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures settings of the arbitrary sequence
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repetitionRate/Sets or queries the number of repetitions of the arbitrary sequence
/// HIPAR channel/Numerical selection of a channel
/// HIPAR endpointBehavior/Sets/Queries the arbitrary endpoint behavior, when arbitrary function
/// HIPAR endpointBehavior/is finished.
ViStatus _VI_FUNC rsngx_ConfigureArbitrarySequenceSettings(ViSession instrSession,
                                                           ViInt32   repetitionRate,
                                                           ViInt32   channel,
                                                           ViInt32   endpointBehavior)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_SEQUENCE_REPETITION_RATE, repetitionRate),
            3, "Repetition Rate");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_SEQUENCE_ENDPOINT_BEHAVIOR, endpointBehavior),
            4, "Endpoint Behavior");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the total number of points of the arbitrary sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR numberofPoints/Queries the total number of points of the arbitrary
/// HIPAR numberofPoints/sequence.Attribute:
ViStatus _VI_FUNC rsngx_QueryArbitrarySequenceNumberOfPoints(ViSession instrSession,
                                                             ViInt32   channel,
                                                             ViInt32*  numberofPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_ARBITRARY_SEQUENCE_NUMBER_OF_POINTS, numberofPoints),
            3, "Number of Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Transfers the defined arbitrary table to the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_TransferArbitrarySequenceTable(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ARBITRARY_SEQUENCE_TRANSFER_TABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configure Analog Input settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Enables or disables the analog input for the selected channel.
/// HIPAR type/Sets or queries the analog input type applied at the analog input of
/// HIPAR type/the terminal block.
ViStatus _VI_FUNC rsngx_ConfigureAnalogInputSettings(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   enabled,
                                                     ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_INPUT_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_INPUT_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the trigger condition of the analog input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR enabled/Enables or disables the trigger condition of the analog input.
ViStatus _VI_FUNC rsngx_ConfigureAnalogInputTriggeredConditionEnabled(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_ANALOG_INPUT_TRIGGERED_CONDITION_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  DIO pin/s are enabled with a mode set to Analog In.
/// HIFN     When DIO pin is enabled with Analog In mode, analog input of the
/// HIFN  channel assigned to that pin will be enabled when the correct voltage
/// HIFN  is applied to the DIO pin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR pin/DIO pin/s are enabled with a mode set to Analog In.
ViStatus _VI_FUNC rsngx_ConfigureAnalogInputTriggerPin(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   pin)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_INPUT_TRIGGER_PIN, pin),
            3, "Pin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  DIO pin/s are enabled with a mode set to Analog In.
/// HIFN     When DIO pin is enabled with Analog In mode, analog input of the
/// HIFN  channel assigned to that pin will be enabled when the correct voltage
/// HIFN  is applied to the DIO pin.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/"Linked Fuses List" control.
/// HIPAR pin/Returns Analog Input pins as a comma seperated string.
ViStatus _VI_FUNC rsngx_QueryAnalogInputTriggerPin(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   bufferSize,
                                                   ViChar    pin[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_QUERY_ANALOG_INPUT_TRIGGER_PIN,
            bufferSize, pin),
            3, "Pin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the analog input adjustment state.
/// HIFN     StateDescriptions
/// HIFN     0-15
/// HIFN       0x0 - 0xE ( 0x0000 - 0x1111) bit3|bit2|bit1|bit0
/// HIFN       bit0 - pin 1 of analog input
/// HIFN       bit1 - pin 2 of analog input
/// HIFN       bit2 - pin 3 of analog input
/// HIFN       bit3 - pin 4 of analog input
/// HIFN       e.g. 15 - All analog input pins are adjusted.
/// HIFN       e.g. 9 - Pin 1 and pin 4 are adjusted.
/// HIFN     16 Idle
/// HIFN     17 Busy
/// HIFN     18 Waiting
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustmentState/Queries the analog input adjustment state.
ViStatus _VI_FUNC rsngx_QueryAnalogInAdjustmentState(ViSession instrSession,
                                                     ViInt32*  adjustmentState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_STATE, adjustmentState),
            2, "Adjustment State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the analog input factory adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_RestoreAnalogInFactoryAdjustment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_FACTORY_RESTORE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns information about previous analog input adjustments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lastAdjustmentDate/Returns the analog input adjustment date (DD-MM-YY).Attribute:
/// HIPAR timesAdjusted/Queries the number of counts performed for analog input adjustment
/// HIPAR timesAdjusted/.Attribute:
ViStatus _VI_FUNC rsngx_QueryAnalogInPreviousAdjustmentsInformation(
                            ViSession instrSession,
                            ViChar    lastAdjustmentDate[],
                            ViInt32*  timesAdjusted)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_DATE,
            RS_MAX_SHORT_MESSAGE_BUF_SIZE, lastAdjustmentDate),
            2, "Last Adjustment Date");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_COUNT, timesAdjusted),
            3, "Times Adjusted");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the analog input pin for adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pin/Selects the analog input pin for adjustment.
ViStatus _VI_FUNC rsngx_ConfigureAnalogInAdjustmentPin(ViSession instrSession,
                                                       ViInt32   pin)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_START, pin),
            2, "Pin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the analog input pin for adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pin/Queries the analog input pin for adjustment.
ViStatus _VI_FUNC rsngx_QueryAnalogInAdjustmentPin(ViSession instrSession,
                                                   ViInt32*  pin)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_QUERY_ANALOG_IN_ADJUSTMENT_START, pin),
            2, "Pin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the analog input adjustment data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustmentValue/Sets the analog input adjustment data.
ViStatus _VI_FUNC rsngx_ConfigureAnalogInAdjustmentValue(ViSession instrSession,
                                                         ViReal64  adjustmentValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_VALUE, adjustmentValue),
            2, "Adjustment Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of high voltage applied to the analog input pin during
/// HIFN  adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SetAnalogInAdjustmentHighVoltage(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_HIGH_VOLTAGE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of low voltage applied to the analog input pin during
/// HIFN  adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SetAnalogInAdjustmentLowVoltage(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_LOW_VOLTAGE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the analog input adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SaveAnalogInAdjustment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_SAVE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Cancels the analog input adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_EndAnalogInAdjustment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_END, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Cancels the analog input adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_CancelAnalogInAdjustment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_ANALOG_IN_ADJUSTMENT_CANCEL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the current state of channel adjustment.
/// HIFN
/// HIFN     State   | Descriptions
/// HIFN     0       | Idle
/// HIFN     1       | Busy
/// HIFN     2       | Waiting
/// HIFN     12      | Voltage adjustment completed
/// HIFN     13      | Current adjustment completed
/// HIFN     16      | Successful channel adjustment
/// HIFN     17      | Failed channel adjustment
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR adjustmentState/Returns the current state of channel adjustment.
ViStatus _VI_FUNC rsngx_QueryChannelAdjustmentState(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32*  adjustmentState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_STATE, adjustmentState),
            3, "Adjustment State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the channel adjustment information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR lastAdjustmentDate/Returns the channel adjustment date.Attribute:
/// HIPAR timesAdjusted/Queries the number of counts performed for channel
/// HIPAR timesAdjusted/adjustment.Attribute:
ViStatus _VI_FUNC rsngx_QueryChannelPreviousAdjustmentInformation(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViChar    lastAdjustmentDate[],
                            ViInt32*  timesAdjusted)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_DATE,
            RS_MAX_SHORT_MESSAGE_BUF_SIZE, lastAdjustmentDate),
            3, "Last Adjustment Date");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_COUNT, timesAdjusted),
            4, "Times Adjusted");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the temperature of selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR temperature/Returns the temperature of selected channel.Attribute:
ViStatus _VI_FUNC rsngx_QueryChannelAdjustmentTemperature(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64* temperature)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_TEMPERATURE, temperature),
            3, "Temperature");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the channel adjustment process.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ConfigureChannelAdjustmentStart(ViSession instrSession,
                                                        ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_START, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the current for channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR currentValue/Set the current for channel adjustment.
ViStatus _VI_FUNC rsngx_ConfigureChannelAdjustmentCurrentValue(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  currentValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_CURRENT_VALUE, currentValue),
            3, "Current Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of low current output during channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_SetChannelAdjustmentCurrentMin(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_CURRENT_MIN, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of high current output during channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_SetChannelAdjustmentCurrentMax(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_CURRENT_MAX, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the voltage for channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR voltageValue/Set the voltage for channel adjustment.
ViStatus _VI_FUNC rsngx_ConfigureChannelAdjustmentVoltageValue(
                            ViSession instrSession,
                            ViInt32   channel,
                            ViReal64  voltageValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_VOLTAGE_VALUE, voltageValue),
            3, "Voltage Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of high voltage output during channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_SetChannelAdjustmentVoltageMax(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_VOLTAGE_MAX, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of low voltage output during channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_SetChannelAdjustmentVoltageMin(ViSession instrSession,
                                                       ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_VOLTAGE_MIN, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_SaveChannelAdjustment(ViSession instrSession,
                                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_SAVE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Cancels the channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_EndChannelAdjustment(ViSession instrSession,
                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_END, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Cancels the channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_CancelChannelAdjustment(ViSession instrSession,
                                                ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_CANCEL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the factory channel adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_RestoreChannelFactoryAdjustment(ViSession instrSession,
                                                        ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_CHANNEL_ADJUSTMENT_FACTORY_RESTORE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the digital output channel selection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel for the digital output source.
/// HIPAR digitalI/OEnabled/Enables or disables the digital output channel selection.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOEnabled(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean digitalIOEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIO:OUTP %ld,%d", channel, digitalIOEnabled ? 1 : 0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the digital output channel state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Channel for the digital output source.
/// HIPAR digitalI/OState/Returns Digital I/O State.
ViStatus _VI_FUNC rsngx_QueryDigitalIOState(ViSession  instrSession,
                                            ViInt32    channel,
                                            ViBoolean* digitalIOState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIO:OUTP:STAT? %ld", channel);
    checkErr(rsngx_QueryViBoolean(instrSession, cmd, digitalIOState));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets channel selection for the digital output fault source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalI/OChannel/Sets channel selection for the digital output fault source.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOFaultChannel(ViSession instrSession,
                                                       ViInt32   faultChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_DIGITAL_IO_FAULT_CHANNEL, faultChannel),
            2, "Fault Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the digital output fault.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR faultEnabled/Enables or disables the digital output fault.
/// HIPAR source/Sets the operation modes of the digital output fault
/// HIPAR source/source.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOFault(ViSession instrSession,
                                                ViBoolean faultEnabled,
                                                ViInt32   sourceMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_DIGITAL_IO_FAULT_ENABLED, faultEnabled),
            2, "Fault Enabled");

    if (faultEnabled == VI_FALSE)
    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_DIGITAL_IO_FAULT_MODE, sourceMode),
            3, "Source Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select digital output fault signal type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signal/Select digital output fault signal type.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOFaultSignal(ViSession instrSession,
                                                      ViInt32   signal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_DIGITAL_IO_FAULT_SIGNAL, signal),
            2, "Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the digital I/O output source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the channel for digital output source.
/// HIPAR outputSource/Selects the digital IO output source.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOOutputSource(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   outputSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  sourceCmdString;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngDigitalIOOutputSource, outputSource, &sourceCmdString));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIO:OUTP:SOUR %ld,%s", channel, sourceCmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select digital output signal type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the channel for digital output source.
/// HIPAR source/Selects the digital IO output source.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOOutputSignal(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  sourceCmdString;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngDigitalIOFaultSignal, source, &sourceCmdString));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIO:OUTP:SIGN %ld,%s", channel, sourceCmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the digital output source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the channel for digital output source.
/// HIPAR digitalOutputSource/Returns Digital Output Source.
ViStatus _VI_FUNC rsngx_QueryDigitalIOOutputSource(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32*  digitalOutputSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rangeTableString[RS_MAX_MESSAGE_LEN] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIO:OUTP:SOUR? %ld", channel);
    checkErr(rsngx_QueryViString(instrSession, cmd, RS_MAX_MESSAGE_LEN, rangeTableString));

    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsNgx_rngDigitalIOOutputSource, rangeTableString, digitalOutputSource));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the specific channel of the battery
/// HIFN  simulator state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR batterySimulatorEnabled/Enables or disables the specific channel of the battery
/// HIPAR batterySimulatorEnabled/simulator state.
ViStatus _VI_FUNC rsngx_ConfigureBatterySimulatorEnabled(ViSession instrSession,
                                                         ViInt32   channel,
                                                         ViBoolean batterySimulatorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_ENABLED, batterySimulatorEnabled),
            3, "Battery Simulator Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the current battery model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rsngx_ResetBatterySimulatorModel(ViSession instrSession,
                                                   ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_RESET_MODEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a battery model for editing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR action/Sets the action with selected file.
/// HIPAR modelName/Saves the current battery model to a file.
ViStatus _VI_FUNC rsngx_ManageBatterySimulatorModelSettings(ViSession instrSession,
                                                            ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSNGX_ATTR_BATTERY_SIMULATOR_SAVE_MODEL, RSNGX_ATTR_BATTERY_SIMULATOR_LOAD_MODEL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, RSNGX_VAL_SAVE, RSNGX_VAL_LOAD),
            2, "Action");

    checkErr(rsngx_SetAttributeViString(instrSession, "", attr[action], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Congiures initial settings for a battery model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR modelName/Sets a filename for the battery model.
/// HIPAR capacity/Sets the battery model capacity.
/// HIPAR initialStateofCharge/Sets the initial state of charge (SoC) of the battery
/// HIPAR initialStateofCharge/model.
ViStatus _VI_FUNC rsngx_ConfigureBatterySimulatorModelStats(ViSession instrSession,
                                                            ViInt32   channel,
                                                            ViString  modelName,
                                                            ViInt32   modelPartition,
                                                            ViReal64  capacity,
                                                            ViReal64  initialStateofCharge)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  rangeString;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngBatterySimulatorModelNamePartition, modelPartition, &rangeString));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "BATT:MOD:FNAM '%s',%s", modelName, rangeString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_MODEL_CAPACITY, capacity),
            4, "Capacity");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_INITIAL_SOC, initialStateofCharge),
            5, "Initial State of Charge");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the battery model data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stateofCharge_SoC/Sets the value for battery state of charge (SoC) of the battery model.
/// HIPAR opencircuitVoltage_VoC/Sets the open-circuit voltage (Voc) of the battery model.
/// HIPAR internalResistance_ESR/Sets the value for internal resistance (ESR) of the battery model.
/// HIPAR numberofvalues/Sets the number of values written.
ViStatus _VI_FUNC rsngx_ConfigureBatterySimulatorModelData(ViSession instrSession,
                                                           ViInt32   stateofCharge_SoC[],
                                                           ViReal64  opencircuitVoltage_VoC[],
                                                           ViReal64  internalResistance_ESR[],
                                                           ViInt32   numberofvalues)
{
    ViStatus error = VI_SUCCESS;
    ViString target = NULL;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    if (numberofvalues > 0)
    {
        checkErr(RsCore_StrCatDynamicString(instrSession, &target, "BATT:MOD:DATA ", NULL, NULL));
        for (i = 0; i < numberofvalues; i++)
        {
            checkErr(RsCore_StrCatDynamicViReal64(instrSession, &target, (i == 0) ? NULL : ",", stateofCharge_SoC[i], NULL));
            checkErr(RsCore_StrCatDynamicViReal64(instrSession, &target, ",", opencircuitVoltage_VoC[i], NULL));
            checkErr(RsCore_StrCatDynamicViReal64(instrSession, &target, ",", internalResistance_ESR[i], NULL));
        }

        checkErr(RsCore_Write(instrSession, target));
        checkErr(rsngx_CheckStatus(instrSession));
    }

Error:
    if (target) free(target);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the battery model data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stateofCharge_SoC/Sets the value for battery state of charge (SoC) of the battery model.
/// HIPAR opencircuitVoltage_VoC/Sets the open-circuit voltage (Voc) of the battery model.
/// HIPAR internalResistance_ESR/Sets the value for internal resistance (ESR) of the battery model.
/// HIPAR returnedValues/Returns the number of values written.
ViStatus _VI_FUNC rsngx_QueryBatterySimulatorModelData(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32   stateofCharge_SoC[],
                                                       ViReal64  opencircuitVoltage_VoC[],
                                                       ViReal64  internalResistance_ESR[],
                                                       ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* readValues = NULL;
    ViInt32   allValues = 0;
    ViInt32   i = 0;
    ViInt32   y = 0;
    ViInt32   count = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "BATT:MOD:DATA?", &readValues, &allValues));

    checkErr(rsngx_CheckStatus(instrSession));

    count = (allValues / 3 > arraySize) ? arraySize : allValues / 3;

    while (count > 0)
    {
        stateofCharge_SoC[i] = (ViInt32)readValues[y++];
        opencircuitVoltage_VoC[i] = readValues[y++];
        internalResistance_ESR[i] = readValues[y++];
        i++;
        count--;
    }

    if (returnedValues) *returnedValues = i;

Error:
    if (readValues) free(readValues);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Transfers the loaded battery model into the channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transfertoChannel/Transfers the loaded battery model into the channel.
ViStatus _VI_FUNC rsngx_TransferBatterySimulatorModel(ViSession instrSession,
                                                      ViInt32   transfertoChannel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_TRANSFER_MODEL, transfertoChannel),
            2, "Transfer to Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for the battery simulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR fullCapacity/Sets the full battery capacity for the specific channel.
/// HIPAR stateofCharge/Sets the state of charge (SoC) of the battery simulator.
ViStatus _VI_FUNC rsngx_ConfigureBatterySimulatorSettings(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64  fullCapacity,
                                                          ViReal64  stateofCharge)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_FULL_CAPACITY, fullCapacity),
            3, "Full Capacity");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_STATE_OF_CHARGE, stateofCharge),
            4, "State of Charge");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current limit of the battery
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR currentLimit/Queries the current limit.
ViStatus _VI_FUNC rsngx_QueryBatterySimulatorCurrentLimit(ViSession instrSession,
                                                          ViInt32   channel,
                                                          ViReal64* currentLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_CURRENT_LIMIT, currentLimit),
            3, "Current Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current condition of the battery.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR batteryStatus/Queries the status of the battery (charging or discharging).
/// HIPAR remainingCapacity/Queries the remaining battery capacity from the specific channel.
/// HIPAR terminalVoltage/Queries the terminal voltage (Vt) of the specific channel.
/// HIPAR internalResistance/Queries the battery simulator internal resistance (ESR).
/// HIPAR current/Queries the current (A) from the specific channel.
ViStatus _VI_FUNC rsngx_QueryBatterySimulatorCondition(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32*  batteryStatus,
                                                       ViReal64* remainingCapacity,
                                                       ViReal64* terminalVoltage,
                                                       ViReal64* internalResistance,
                                                       ViReal64* current)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_QUERY_STATUS, batteryStatus),
            3, "Battery Status");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_REMAINING_CAPACITY, remainingCapacity),
            4, "Remaining Capacity");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_TERMINAL_VOLTAGE, terminalVoltage),
            5, "Terminal Voltage");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_INTERNAL_RESISTANCE, internalResistance),
            6, "Internal Resistance");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_CURRENT, current),
            7, "Current");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the open circuit voltage (Voc) stats for battery simulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR opencircuitVoltage/Queries the open circuit voltage (Voc) of the specific channel.
/// HIPAR emptyOpencircuitVoltage/Queries the open circuit voltage (Voc) for empty SoC, i.e SoC = 0 %.
/// HIPAR fullOpencircuitVoltage/Queries the open circuit voltage (Voc) for full SoC, i.e SoC = 100 %.
ViStatus _VI_FUNC rsngx_QueryBatterySimulatorVOCStats(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViReal64* opencircuitVoltage,
                                                      ViReal64* emptyOpencircuitVoltage,
                                                      ViReal64* fullOpencircuitVoltage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SELECT_CHANNEL, channel),
            2, "Channel");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_VOC, opencircuitVoltage),
            3, "Open-circuit Voltage");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_VOC_EMPTY, emptyOpencircuitVoltage),
            4, "Empty Open-circuit Voltage");

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_VOC_FULL, fullOpencircuitVoltage),
            5, "Full Open-circuit Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  configures the battery simulator limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR endofCharge/Sets or queries the current limit of the battery simulator at
/// HIPAR endofCharge/end-of-charge.
/// HIPAR endofDischarge/Sets or queries the current limit of the battery simulator at
/// HIPAR endofDischarge/end-of-discharge.
/// HIPAR regularChargeLevel/Sets or queries the current limit of the battery simulator at regular
/// HIPAR regularChargeLevel/charge level.
ViStatus _VI_FUNC rsngx_ConfigureBatterySimulatorLimits(ViSession instrSession,
                                                        ViReal64  endofCharge,
                                                        ViReal64  endofDischarge,
                                                        ViReal64  regularChargeLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_CURRENT_LIMIT_END_OF_CHARGE, endofCharge),
            2, "End Of Charge");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_CURRENT_LIMIT_END_OF_DISCHARGE, endofDischarge),
            3, "End Of Discharge");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_SIMULATOR_CURRENT_LIMIT_REGULAR_CHARGE_LEVEL, regularChargeLevel),
            4, "Regular Charge Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  configures the battery model limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR endofCharge/Sets or queries the current limit of the battery model at
/// HIPAR endofCharge/end-of-charge.
/// HIPAR endofDischarge/Sets or queries the current limit of the battery simulator at
/// HIPAR endofDischarge/end-of-discharge.
/// HIPAR regularChargeLevel/Sets or queries the current limit of the battery simulator at regular
/// HIPAR regularChargeLevel/charge level.
ViStatus _VI_FUNC rsngx_ConfigureBatteryModelLimits(ViSession instrSession,
                                                    ViReal64  endofCharge,
                                                    ViReal64  endofDischarge,
                                                    ViReal64  regularChargeLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_MODEL_CURRENT_LIMIT_END_OF_CHARGE, endofCharge),
            2, "End Of Charge");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_MODEL_CURRENT_LIMIT_END_OF_DISCHARGE, endofDischarge),
            3, "End Of Discharge");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_BATTERY_MODEL_CURRENT_LIMIT_REGULAR_CHARGE_LEVEL, regularChargeLevel),
            4, "Regular Charge Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the display brightness and front panel key brightness.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR brightness/Sets the display brightness.
/// HIPAR frontPanelKeyBrightness/Sets the front panel key brightness.
ViStatus _VI_FUNC rsngx_ConfigureBrightness(ViSession instrSession,
                                            ViReal64  brightness,
                                            ViReal64  frontPanelKeyBrightness)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_DISPLAY_BRIGHTNESS, brightness),
            2, "Brightness");

    viCheckParm(rsngx_SetAttributeViReal64(instrSession, "", RSNGX_ATTR_DISPLAY_FRONT_PANEL_KEY_BRIGHTNESS, frontPanelKeyBrightness),
            3, "Front Panel Key Brightness");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR text/Displays a text message box on the front display.
ViStatus _VI_FUNC rsngx_ShowMessageBox(ViSession instrSession,
                                       ViString  text)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_DISPLAY_TEXT, text),
            2, "Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ClearMessageBox(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_DISPLAY_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables touch interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR touchscreenEnabled/Enables or disables touch interface.
ViStatus _VI_FUNC rsngx_ConfigureSystemTouchEnabled(ViSession instrSession,
                                                    ViInt32   touchscreenEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SYSTEM_TOUCH_ENABLED, touchscreenEnabled),
            2, "Touchscreen Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Contains settings for the trigger system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerEnabled/Enables or disables the trigger system.
/// HIPAR source/Sets the trigger source.
/// HIPAR outputChannel/Sets the device channel for trigger source Output.
/// HIPAR operationMode/Sets the operation mode to trigger on for trigger source
/// HIPAR operationMode/operation mode.
ViStatus _VI_FUNC rsngx_ConfigureTriggerSettings(ViSession instrSession,
                                                 ViBoolean triggerEnabled,
                                                 ViInt32   source,
                                                 ViInt32   outputChannel,
                                                 ViInt32   operationMode)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSNGX_ATTR_TRIGGER_OUTPUT_CHANNEL,
             RSNGX_ATTR_TRIGGER_OPERATION_MODE_CHANNEL,
             RSNGX_ATTR_TRIGGER_DIO_CHANNEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_TRIGGER_ENABLED, triggerEnabled),
            2, "Trigger Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_TRIGGER_SOURCE, source),
            3, "Source");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", attr[source], outputChannel),
            4, "Output Channel");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_TRIGGER_OPERATION_MODE, operationMode),
            5, "Operation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the DIO pin to trigger on for trigger source Digital In Channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR DIOPin/Sets the DIO pin to trigger on for trigger source Digital In Channel.
ViStatus _VI_FUNC rsngx_ConfigureTriggerDIOPin(ViSession instrSession,
                                               ViInt32   DIOPin)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_TRIGGER_DIO_PIN, DIOPin),
            2, "DIO Pin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of external trigger input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSlope/Sets the type of external trigger input.
ViStatus _VI_FUNC rsngx_ConfigureTriggerSlope(ViSession instrSession,
                                              ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_TRIGGER_SLOPE, triggerSlope),
            2, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the enable state of the master on/off of Digital I/O
/// HIFN  trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets or queries the enable state of the master on/off of Digital I/O
/// HIPAR enabled/trigger.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerEnabled(ViSession instrSession,
                                                         ViInt32   enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_DIGITAL_IO_TRIGGER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures state and direction of the specified Digital I/O line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Numerical selection of a Digital I/O Line (Pin).
/// HIPAR enabled/Sets or queries the enable state of the specified Digital I/O line.
/// HIPAR direction/Sets or queries the specified Digital I/O line to function as Trigger
/// HIPAR direction/Input/Output.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerLine(ViSession instrSession,
                                                      ViInt32   line,
                                                      ViInt32   enabled,
                                                      ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, 1, 8),
            2, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_LOGIC,
            "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Line");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_DIRECTION, direction),
            4, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the trigger condition of the specified Digital I/O line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Numerical selection of a Digital I/O Line (Pin).
/// HIPAR mode/Sets the trigger condition of the specified Digital I/O line.
/// HIPAR value/If required (in modes like PLEVEL, ILEVEL...), enter value to set the
/// HIPAR value/property (power, current...).
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerLineMode(ViSession instrSession,
                                                          ViInt32   line,
                                                          ViInt32   mode,
                                                          ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  lineCount = 0;
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Line", &lineCount),
        2, "Line");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, 1, lineCount),
            2, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_HIDDEN_DIGITAL_IO_TRIGGER_LINE_MODE,
            "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Line");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsNgx_rngDigitalTriggerIOLineMode, mode, &p2string),
        3, "Mode");

    switch (mode)
    {
    case RSNGX_VAL_DIGITAL_TRIGGER_MODE_VLEVEL:
    case RSNGX_VAL_DIGITAL_TRIGGER_MODE_ILEVEL:
    case RSNGX_VAL_DIGITAL_TRIGGER_MODE_PLEVEL:
        (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s,%lf", p2string, value);
        break;

    default:
        (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s", p2string);
        break;
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, repCap, RSNGX_ATTR_HIDDEN_DIGITAL_IO_TRIGGER_LINE_MODE, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the trigger logic (Active High/Active Low) of the
/// HIFN  specified Digital I/O line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Numerical selection of a Digital I/O Line (Pin).
/// HIPAR activeLevel/Sets or queries the trigger logic (Active High/Active Low) of the
/// HIPAR activeLevel/specified Digital I/O line.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerLineLogic(ViSession instrSession,
                                                           ViInt32   line,
                                                           ViInt32   activeLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, 1, 8),
            2, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_LOGIC,
            "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Line");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_LOGIC, activeLevel),
            3, "Active Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the trigger channel of the specified Digital I/O line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Numerical selection of a Digital I/O Line (Pin).
/// HIPAR channel/Sets the trigger channel of the specified Digital I/O line.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerLineChannel(ViSession instrSession,
                                                             ViInt32   line,
                                                             ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, 1, 8),
            2, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_LOGIC,
            "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Line");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_CHANNEL, channel),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the enable state of the specified Digital I/O line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Numerical selection of a Digital I/O Line (Pin).
/// HIPAR enabled/Sets or queries the enable state of the specified Digital I/O line.
ViStatus _VI_FUNC rsngx_ConfigureDigitalIOTriggerLineDIOEnabled(
                            ViSession instrSession,
                            ViInt32   line,
                            ViInt32   enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, line, 1, 8),
            2, "Line");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_LOGIC,
            "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Line");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, repCap, RSNGX_ATTR_DIGITAL_IO_TRIGGER_LINE_DIO_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the actual display content (screenshot).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destination/Destination of the file for the screenshot to be saved.
ViStatus _VI_FUNC rsngx_ImageSnapshot(ViSession instrSession,
                                      ViString  destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (!destination)
    viCheckParm(RsCore_InvalidNullPointer(instrSession, destination), 2, "Destination");

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, "HCOP:DATA?", destination, RS_VAL_TRUNCATE));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the width and height of the screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageWidth/Returns the horizontal expansion of the screenshots.
/// HIPAR imageHeight/Returns the vertical expansion of the screenshots.
ViStatus _VI_FUNC rsngx_QueryImageSize(ViSession instrSession,
                                       ViInt32*  imageWidth,
                                       ViInt32*  imageHeight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_IMAGE_WIDTH, imageWidth),
            2, "Image Width");

    viCheckParm(rsngx_GetAttributeViInt32(instrSession, "", RSNGX_ATTR_IMAGE_HEIGHT, imageHeight),
            3, "Image Height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the GPIB interface information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interfaceInfo/Queries the GPIB interface information.
ViStatus _VI_FUNC rsngx_QueryGPIBInterfaceInfo(ViSession instrSession,
                                               ViUInt32  bufferSize,
                                               ViChar    interfaceInfo[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_GPIB_INTERFACE_INFO,
            bufferSize, interfaceInfo),
            3, "Interface Info");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the available system interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR availableSystemInterface/Queries the available system interface.
ViStatus _VI_FUNC rsngx_QueryAvailableSystemInterface(ViSession instrSession,
                                                      ViUInt32  bufferSize,
                                                      ViChar    availableSystemInterface[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViString(instrSession, "", RSNGX_ATTR_AVAILABLE_SYSTEM_INTERFACE,
            bufferSize, availableSystemInterface),
            3, "Available System Interface");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the LAN interface settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dhcpEnabled/Sets the LAN interface mode.
/// HIPAR ipAddress/Sets or queries IP address of the LAN interface.
/// HIPAR subnetMask/Sets or Queries the subnet mask for LAN.
/// HIPAR gateway/Sets or queries gateway for LAN.
ViStatus _VI_FUNC rsngx_ConfigureLANNetworkSettings(ViSession instrSession,
                                                    ViInt32   dhcpEnabled,
                                                    ViString  ipAddress,
                                                    ViString  subnetMask,
                                                    ViString  gateway)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "NGP"))
    {
        viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SOCKET_DHCP_ENABLED, dhcpEnabled),
                2, "DHCP Enabled");

        if (dhcpEnabled == VI_FALSE)
        {
            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SOCKET_IP_ADDRESS, ipAddress),
                    3, "IP Address");

            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SOCKET_MASK, subnetMask),
                    4, "Subnet Mask");

            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SOCKET_GATEWAY, gateway),
                    5, "Gateway");
        }
    }
    else
    {
        viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_LAN_DHCP_ENABLED, dhcpEnabled),
                2, "DHCP Enabled");

        if (dhcpEnabled == VI_FALSE)
        {
            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_LAN_IP_ADDRESS, ipAddress),
                    3, "IP Address");

            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_LAN_MASK, subnetMask),
                    4, "Subnet Mask");

            viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_LAN_DEFAULT_GATEWAY, gateway),
                    5, "Gateway");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the USB class.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usbClass/Sets the USB class.
ViStatus _VI_FUNC rsngx_ConfigureInterfaceUSBClass(ViSession instrSession,
                                                   ViInt32   usbClass)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_INTERFACE_USB_CLASS, usbClass),
            2, "USB Class");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Apply LAN configuration settings.

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ApplyLANNetworkSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SOCKET_APPLY, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Discard LAN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_DiscardLANSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "NGP"))
    {
        checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SOCKET_DISCARD, NULL));
    }
    else
    {
        checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_LAN_DISCARD, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reset instrument LAN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ResetLANSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "NGP"))
    {
        checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SOCKET_RESET, NULL));
    }
    else
    {
        checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_LAN_RESET, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries instrument hostname.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hostname/Sets or queries instrument hostname.
ViStatus _VI_FUNC rsngx_ConfigureLANHostname(ViSession instrSession,
                                             ViString  hostname)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_LAN_HOSTNAME, hostname),
            2, "Hostname");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries MAC address of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR macAddress/Queries MAC address of the instrument.
ViStatus _VI_FUNC rsngx_QueryLANMACAddress(ViSession instrSession,
                                           ViChar    macAddress[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   instrumentResponse[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSNGX_ATTR_LAN_MAC_ADDRESS, RS_VAL_DIRECT_USER_CALL, RS_MAX_SHORT_MESSAGE_BUF_SIZE, instrumentResponse));

    strncpy(macAddress, instrumentResponse, RS_MAX_SHORT_MESSAGE_BUF_SIZE);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the edit mode of LAN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR editedMode/Queries the edit mode of LAN settings.
ViStatus _VI_FUNC rsngx_QueryLANEditedMode(ViSession  instrSession,
                                           ViBoolean* editedMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_LAN_EDITED_MODE, editedMode),
            2, "Edited Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables WLAN state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables WLAN state.
ViStatus _VI_FUNC rsngx_ConfigureWLANEnabled(ViSession instrSession,
                                             ViInt32   enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_WLAN_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Connects or disconnects WLAN to the predefined wireless access point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectionEnabled/Connects or disconnects WLAN to the predefined wireless access point.
ViStatus _VI_FUNC rsngx_ConfigureWLANConnectionEnabled(ViSession instrSession,
                                                       ViInt32   connectionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_WLAN_CONNECTION_ENABLED, connectionEnabled),
            2, "Connection Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries SSID of the access point when wireless interface works
/// HIFN  as a client.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ssid/Sets or queries SSID of the access point when wireless interface works
/// HIPAR ssid/as a client.
ViStatus _VI_FUNC rsngx_ConfigureWLANSSID(ViSession instrSession,
                                          ViString  ssid)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_WLAN_SSID, ssid),
            2, "SSID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries password for WLAN.

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR password/Sets or queries password for WLAN.
ViStatus _VI_FUNC rsngx_ConfigureWLANPassword(ViSession instrSession,
                                              ViString  password)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_WLAN_PASSWORD, password),
            2, "Password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures network settings for WLAN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dhcpEnabled/Enables or disables DHCP for WLAN.
/// HIPAR reserved/Reserved for future use.
/// HIPAR subnetMask/Sets or queries subnet mask for WLAN.
/// HIPAR gateway/Sets or queries gateway address for WLAN.
ViStatus _VI_FUNC rsngx_ConfigureWLANNetworkSettings(ViSession instrSession,
                                                     ViInt32   dhcpEnabled,
                                                     ViString  reserved,
                                                     ViString  subnetMask,
                                                     ViString  gateway)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_WLAN_DHCP_ENABLED, dhcpEnabled),
            2, "DHCP Enabled");

    if (dhcpEnabled == 0)
    {
        viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_WLAN_MASK, subnetMask),
                4, "Subnet Mask");

        viCheckParm(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_WLAN_GATEWAY, gateway),
                5, "Gateway");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries IP address for WLAN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Queries IP address for WLAN.
ViStatus _VI_FUNC rsngx_QueryWLANIPAddress(ViSession instrSession,
                                           ViChar    ipAddress[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   instrumentResponse[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RSNGX_ATTR_WLAN_IP_ADDRESS, RS_VAL_DIRECT_USER_CALL, RS_MAX_SHORT_MESSAGE_BUF_SIZE, instrumentResponse));

    strncpy(ipAddress, instrumentResponse, RS_MAX_SHORT_MESSAGE_BUF_SIZE);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies WLAN configuration settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ApplyWLANNetworkSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_WLAN_APPLY, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  Configures VNC settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR vnc/Enables or disables VNC connection.
/// HIPAR port/Sets or queries instrument VNC port.
ViStatus _VI_FUNC rsngx_ConfigureVNCSettings(ViSession instrSession,
                                             ViBoolean vnc,
                                             ViInt32   port)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "NGP"))
    {
        viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_VNC_ENABLED_NGP, vnc),
                2, "VNC");

        viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_VNC_PORT_NGP, port),
                3, "Port");
    }
    else
    {
        viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_VNC_ENABLED, vnc),
                2, "VNC");

        viCheckParm(rsngx_SetAttributeViInt32(instrSession, NULL, RSNGX_ATTR_VNC_PORT, port),
                3, "Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the sound settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorBeepEnabled/Activates/Deactivates or Queries the beeper tone.
/// HIPAR trippedBeepEnabled/Sets 'Protection' beeper tone.
/// HIPAR continuousBeepEnabled/Sets 'current control' beeper tone. If the 'CC-Mode Continuous Beep'
/// HIPAR continuousBeepEnabled/is activated, a continue beep sound alert when the selected output
/// HIPAR continuousBeepEnabled/channel goes into CC mode.
ViStatus _VI_FUNC rsngx_ConfigureSoundBeeperSettings(ViSession instrSession,
                                                     ViBoolean errorBeepEnabled,
                                                     ViInt32   trippedBeepEnabled,
                                                     ViInt32   continuousBeepEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_BEEPER_ENABLED, errorBeepEnabled),
            2, "Error Beep Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SOUND_PROTECTION_TRIPPED_BEEP_ENABLED, trippedBeepEnabled),
            3, "Tripped Beep Enabled");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", RSNGX_ATTR_SOUND_CC_MODE_CONTINUOUS_BEEP_ENABLED, continuousBeepEnabled),
            4, "Continuous Beep Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a single 'protection' beep immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_GetSoundBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SOUND_BEEP, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  Enables or disables beep sound for operation complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR completeBeep/Enables or disables beep sound for operation complete.
ViStatus _VI_FUNC rsngx_ConfigureSoundCompleteBeepEnabled(ViSession instrSession,
                                                          ViBoolean completeBeep)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_SOUND_COMPLETE_BEEP_ENABLED, completeBeep),
            2, "Complete Beep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sent a single operation complete beep immediately
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_GetSoundCompleteBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SOUND_COMPLETE_BEEP_IMMEDIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries warning  beeper tone state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR warningBeep/Sets or queries warning  beeper tone state.
ViStatus _VI_FUNC rsngx_ConfigureSoundWarningBeepEnabled(ViSession instrSession,
                                                         ViBoolean warningBeep)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_SOUND_WARNING_BEEP_ENABLED, warningBeep),
            2, "Warning Beep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a single error/warning beep immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_GetSoundWarningBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SOUND_WARNING_BEEP_IMMEDIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries output beeper tone state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR outputBeep/Sets or queries output beeper tone state.
ViStatus _VI_FUNC rsngx_ConfigureSoundOutputBeepEnabled(ViSession instrSession,
                                                        ViBoolean outputBeep)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, NULL, RSNGX_ATTR_SOUND_OUTPUT_BEEP_ENABLED, outputBeep),
            2, "Output Beep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates settings file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Creates settings file.
ViStatus _VI_FUNC rsngx_SaveSystemSettingsFile(ViSession instrSession,
                                               ViString  path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SYSTEM_SAVE_SETTINGS_FILE, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restarts the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_RestartInstrument(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, NULL, RSNGX_ATTR_SYSTEM_RESTART, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries system uptime.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR uptime/Queries system uptime.
ViStatus _VI_FUNC rsngx_QuerySystemUptime(ViSession instrSession,
                                          ViReal64* uptime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_GetAttributeViReal64(instrSession, NULL, RSNGX_ATTR_SYSTEM_UPTIME, uptime),
            2, "Uptime");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of ID Response Query.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
/// HIPAR returnedValues/Returns length of string readed from instrument.
ViStatus _VI_FUNC rsngx_IDQueryResponse(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    IDQueryResponse[],
                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "*IDN?", bufferSize, IDQueryResponse, returnedValues));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rsngx_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIFN Note that the actual timeout used may be higher than the one
/// HIFN requested.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Specifies the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
/// HIPAR VISATimeout/Note that the actual timeout used may be higher than the one
/// HIPAR VISATimeout/requested.
ViStatus _VI_FUNC rsngx_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Queries the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
ViStatus _VI_FUNC rsngx_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/******************************************************************************************************
* Function: rsngx_SetOPCTimeout
* Purpose:  Sets OPC timeout
*****************************************************************************************************/
ViStatus _VI_FUNC rsngx_SetOPCTimeout(ViSession instrSession,
                                      ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32)timeout));

Error:
    return error;
}

/******************************************************************************************************
* Function: rsngx_GetOPCTimeout
* Purpose:  Return OPC timeout
*****************************************************************************************************/
ViStatus _VI_FUNC rsngx_GetOPCTimeout(ViSession instrSession,
                                      ViUInt32* timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32*)timeout));

Error:
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsngx_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/*****************************************************************************
* Function: rsngx_SetAttribute<type> Functions
* Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
*           ViInt32, ViReal64, ViString, ViBoolean, and ViSession data types.
*****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rsngx_SetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */
ViStatus _VI_FUNC rsngx_SetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsngx_SetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsngx_SetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* RawString */
ViStatus _VI_FUNC rsngx_SetAttributeRawString(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsngx_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
* Function: rsngx_GetAttribute<type> Functions
* Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
*           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
*****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsngx_GetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */

ViStatus _VI_FUNC rsngx_GetAttributeViInt64(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeId,
                                            ViInt64*  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsngx_GetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsngx_GetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeId,
                                             ViInt32   bufferSize,
                                             ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* RawString */
ViStatus _VI_FUNC rsngx_GetAttributeRawString(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeId,
                                              ViInt32   bufferSize,
                                              ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsngx_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   repeatedCapabilityName,
                                              ViUInt32   attributeId,
                                              ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
/// HIPAR responseLength/The length in bytes of the response read from the instrument. If
/// HIPAR responseLength/Response Length > allocated Buffer Size, some data from the instrument
/// HIPAR responseLength/was lost; resize your buffer to at least Response Length and call again
/// HIPAR responseLength/to capture the missing data from the instrument.
ViStatus _VI_FUNC rsngx_GetAttributeViStringWithRealLen(ViSession instrSession,
                                                        ViString  repeatedCapability,
                                                        ViAttr    attributeId,
                                                        ViInt32   bufferSize,
                                                        ViChar    attributeValue[],
                                                        ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringWithRealLen(instrSession, repeatedCapability, attributeId,
        RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue, responseLength);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a RawString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
/// HIPAR repeatedCapability/If the attribute requires repeated capabilities to be specified, this
/// HIPAR repeatedCapability/parameter specifies one or more comma-separated repeated capability
/// HIPAR repeatedCapability/identifiers on which to set the value of the attribute. If the
/// HIPAR repeatedCapability/attribute does not contain any repeated capabilities, then pass NULL or
/// HIPAR repeatedCapability/an empty string.
/// HIPAR responseLength/The length in bytes of the response read from the instrument. If
/// HIPAR responseLength/Response Length > allocated Buffer Size, some data from the instrument
/// HIPAR responseLength/was lost; resize your buffer to at least Response Length and call again
/// HIPAR responseLength/to capture the missing data from the instrument.
ViStatus _VI_FUNC rsngx_GetAttributeRawStringWithRealLen(ViSession instrSession,
                                                         ViString  repeatedCapability,
                                                         ViAttr    attributeId,
                                                         ViInt32   bufferSize,
                                                         ViChar    attributeValue[],
                                                         ViInt32*  responseLength)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawStringWithRealLen(instrSession, repeatedCapability, attributeId,
        RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue, responseLength);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsngx_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: Rs_GetAttributeRepCapNameId
* Purpose:  This function returns the repeated capability id(s) that
*           belongs to the attribute you specify. More then one id is listed
*           in comma separated string.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeId, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
* Function: Get Attribute Repeated Capability Id Name(s)
* Purpose:  This function returns the repeated capability name(s) that
*           belongs to the attribute and single repeated capability id you
*           may specify. More then one name is listed in comma separated
*           string.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeId,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsngx_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rsngx_DefaultInstrSetup(instrSession));
    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_self_test(ViSession instrSession,
                                  ViInt16*  testResult,
                                  ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the beeper tone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR beeperState/Disables or enables the beeper tone.
ViStatus _VI_FUNC rsngx_BeeperEnabled(ViSession instrSession,
                                      ViBoolean beeperState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsngx_SetAttributeViBoolean(instrSession, "", RSNGX_ATTR_BEEPER_ENABLED, beeperState),
            2, "Beeper State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Local operating mode (front panel control).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SystemLocal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SYSTEM_LOCAL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Remote control (front panel control locked).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SystemRemote(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SYSTEM_REMOTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The front panel control is locked; with the SystemLocal method the
/// HIFN  front panel control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsngx_SystemRWLock(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsngx_SetAttributeViString(instrSession, "", RSNGX_ATTR_SYSTEM_RWLOCK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores or recalls the instrument settings under the specified number
/// HIFN  in an internal memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operation/Selects the operation.
/// HIPAR internalMemory/Specifies intermediate memory
ViStatus _VI_FUNC rsngx_ManageInstrumentSettings(ViSession instrSession,
                                                 ViInt32   operation,
                                                 ViInt32   internalMemory)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSNGX_ATTR_SAVE, RSNGX_ATTR_RECALL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSNGX_VAL_SAVE, RSNGX_VAL_RECALL),
            2, "Operation");

    viCheckParm(rsngx_SetAttributeViInt32(instrSession, "", attr[operation], internalMemory),
            3, "Internal Memory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
    return rsngx_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  This function sets the Enable of selected registers.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     (1) For detailed description of Status Reporting System see also
/// HIFN  operating manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/This control selects the registers that will be configured.
/// HIPAR registerType/This control selects the registers that will be configured.
/// HIPAR registerValue/Sets content of selected register.
ViStatus _VI_FUNC rsngx_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerGroup,
                                          ViInt32   registerType,
                                          ViInt32   registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2registerGroup;
    ViChar*  p2registerType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerGroup, RSNGX_VAL_REGISTER_GROUP_OPER_INSTR, RSNGX_VAL_REGISTER_GROUP_QUES_INSTR_SUMMARY_CHAN_2),
            2, "Register Group");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerType, RSNGX_VAL_REGISTER_TYPE_ENAB, RSNGX_VAL_REGISTER_TYPE_NTR),
            3, "Register Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsngx_rngRegisterGroup, registerGroup, &p2registerGroup),
                2, "Register Group");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsngx_rngRegisterType, registerType, &p2registerType),
                3, "Register Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s %ld", p2registerGroup, p2registerType, registerValue);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status reporting system.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     For detailed description of Status Reporting System see also operating
/// HIFN  manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/This control selects the registers that will be queried.
/// HIPAR registerType/This control selects the registers that will be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsngx_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerGroup,
                                          ViInt32   registerType,
                                          ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2registerGroup;
    ViChar*  p2registerType;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerGroup, RSNGX_VAL_REGISTER_GROUP_OPER_INSTR, RSNGX_VAL_REGISTER_GROUP_QUES_INSTR_SUMMARY_CHAN_2),
            2, "Register Group");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerType, RSNGX_VAL_REGISTER_TYPE_COND, RSNGX_VAL_REGISTER_TYPE_NTR),
            3, "Register Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsngx_rngRegisterGroup, registerGroup, &p2registerGroup),
                2, "Register Group");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsngx_rngRegisterType, registerType, &p2registerType),
                3, "Register Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s?", p2registerGroup, p2registerType);
    checkErr(rsngx_QueryViInt32(instrSession, cmd, registerValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rscma_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscma_init or
/// HIPAR instrSession/rscma_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsngx_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscma_init or
/// HIPAR instrSession/rscma_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsngx_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}


/// HIFN  Returns an array of all the installed instrument options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR options/Returns all the installed instrument options.
/// HIPAR bytesRead/Returnes bytes read.
ViStatus _VI_FUNC rsngx_InstrumentOptions(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    options[],
                                          ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "*OPT?", bufferSize, options, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns true if the instrument has the entered option installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsngx_init or
/// HIPAR instrSession/rsngx_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR option/The option to check on the instrument. E.g., "K10"
/// HIPAR installed/Returns true if the instrument has the entered option installed.
ViStatus _VI_FUNC rsngx_HasOptionInstalled(ViSession  instrSession,
                                           ViString   option,
                                           ViBoolean* installed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    *installed = RsCore_HasInstrumentOptions(instrSession, option);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_GetError(ViSession instrSession,
                                 ViStatus* errorCode,
                                 ViInt32   bufferSize,
                                 ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsngx_utility_error_message, errorCode, bufferSize, description));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rsngx_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/*****************************************************************************
 * Function: viWrite Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write commands directly.
 *****************************************************************************/

ViStatus _VI_FUNC rsngx_Write(ViSession instrSession,
                              ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, writeBuffer));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: Query ViBoolean
* Purpose:  This function queries the ViBoolean value.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2,
                "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: Query ViInt32
* Purpose:  This function queries the ViInt32 value.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2,
                "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
* Function: Query ViReal64
* Purpose:  This function queries the ViReal64 value.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2,
                "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
* Function: Query ViString
* Purpose:  This function queries the ViString value.
*****************************************************************************/
ViStatus _VI_FUNC rsngx_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViInt32   bufferSize,
                                      ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2,
                "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &pbuffer));

    checkErr(rsngx_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsngx_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));
    checkErr(rsngx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsngx_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rsngx_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
